require "rake"
require "fileutils"
require "dtc_rake/config"
require "dtc_rake/product"
require "dtc_rake/ui"
require "dtc_rake/util"

include DtcRake::UI
product = DtcRake::Product.instance
config = DtcRake::Config.instance

if config.upload_uucloud_descriptor
  namespace :upload do
    desc <<-DESC.gsub(/^ {4}/, '')
      Uploads uuCloud deployment descriptor to appbox artifact; requires path to password file.

      rake upload:descriptor DTC_RAKE_PASSWD=12-345-6
      ("~/.uu/12-345-6" will be used)

      rake upload:descriptor DTC_RAKE_PASSWD=/path/to/12-345-6
      (password file from the specified path will be used)
    DESC
    task :descriptor do
      error("Specify password file in env variable DTC_RAKE_PASSWD, e.g. rake upload:descriptor DTC_RAKE_PASSWD=12-345-6") unless ENV["DTC_RAKE_PASSWD"]

      the_file = config.uucloud_descriptor_path
      error("File #{the_file} does not exist") unless File.file?(the_file)

      art_uri = product.appbox_uri
      attch_code = product.uucloud_descriptor_attachment_code
      announce "Uploading #{the_file} to #{art_uri}:#{attch_code}..."
      attch_uri = DtcRake.upload_pack(
        file: the_file,
        appbox_uri: art_uri,
        attachment_code: attch_code,
        credentials: ENV["DTC_RAKE_PASSWD"]
      )
      success "File #{the_file} uploaded to #{attch_uri}"
    end
  end
end
